@@ -17,7 +17,6 @@
 # You should have received a copy of the GNU General Public License
 # along with mb-netmgmt. If not, see <https://www.gnu.org/licenses/
 
-import re
 from socketserver import BaseRequestHandler
 from socketserver import TCPServer as Server
 
@@ -26,35 +25,36 @@ from ncclient.manager import connect
 from ncclient.transport.parser import DefaultXMLParser
 from ncclient.transport.session import (
     HelloHandler,
+    NetconfBase,
     SessionListener,
     qualify,
     sub_ele,
     to_ele,
     to_xml,
 )
-from ncclient.transport.ssh import BUF_SIZE, MSG_DELIM, PORT_NETCONF_DEFAULT, SSHSession
+from ncclient.transport.ssh import MSG_DELIM, PORT_NETCONF_DEFAULT, SSHSession
 
 from mb_netmgmt.__main__ import Protocol
 from mb_netmgmt.ssh import accept
 
 stopped = False
-message_id_regex = ' message-id="([^"]*)"'
 
 
 class Handler(BaseRequestHandler, Protocol):
-    def __init__(self, request, client_address, server):
+    def setup(self):
         session = SSHSession(DefaultDeviceHandler())
         session.add_listener(Listener(super().handle_request))
         self.parser = DefaultXMLParser(session)
-        super().__init__(request, client_address, server)
+        self.session = session
 
     def handle(self):
         self.callback_url = self.server.callback_url
         self.channel = accept(self.request)
         self.open_upstream()
+        self.session._connected = True
         self.handle_prompt()
-        while not stopped:
-            self.read_message()
+        self.session._channel = self.channel
+        self.session.run()
 
     def open_upstream(self):
         to = self.get_to()
@@ -66,10 +66,13 @@ class Handler(BaseRequestHandler, Protocol):
             username=to.username,
             password=to.password,
             hostkey_verify=False,
+            timeout=60,
         )
 
     def handle_prompt(self):
-        hello = to_ele(HelloHandler.build(["urn:ietf:params:netconf:base:1.0"], None))
+        hello = to_ele(
+            HelloHandler.build(DefaultDeviceHandler._BASE_CAPABILITIES, None)
+        )
 
         # A server sending the <hello> element MUST include a <session-id>
         # element containing the session ID for this NETCONF session.
@@ -78,26 +81,22 @@ class Handler(BaseRequestHandler, Protocol):
         session_id.text = "1"
 
         self.channel.sendall(to_xml(hello) + MSG_DELIM.decode())
-        self.read_message()
+
+        def init_cb(id, capabilities):
+            if "urn:ietf:params:netconf:base:1.1" in capabilities:
+                self.session._base = NetconfBase.BASE_11
+
+        self.session.add_listener(HelloHandler(init_cb, None))
 
     def read_proxy_response(self):
-        return {
-            "response": replace_message_id(self.rpc_reply.xml, "") + MSG_DELIM.decode()
-        }
+        return {"response": replace_message_id(self.rpc_reply.xml, "")}
 
     def send_upstream(self, request, request_id):
-        self.rpc_reply = self.manager.rpc(
-            to_ele(request["command"][: -len(MSG_DELIM)])[0]
-        )
-
-    def read_message(self):
-        message = self.channel.recv(BUF_SIZE)
-        self.parser.parse(message)
-        return b""
+        self.rpc_reply = self.manager.rpc(to_ele(request["command"])[0])
 
     def respond(self, response, request_id):
-        response = replace_message_id(response["response"], request_id)
-        self.channel.sendall(response)
+        message = replace_message_id(response["response"], request_id)
+        self.session.send(message)
 
 
 class Listener(SessionListener):
@@ -108,17 +107,11 @@ class Listener(SessionListener):
         tag, attrs = root
         if (tag == qualify("hello")) or (tag == "hello"):
             return
-        request = {"command": replace_message_id(raw, "") + MSG_DELIM.decode()}
-        request_id = get_message_id(raw)
-        self.handle_request(request, request_id)
-
-
-def get_message_id(rpc):
-    try:
-        return re.findall(message_id_regex, rpc)[0]
-    except IndexError:
-        return None
+        request = {"command": replace_message_id(raw, "")}
+        self.handle_request(request, attrs["message-id"])
 
 
 def replace_message_id(rpc, message_id):
-    return re.sub(message_id_regex, f' message-id="{message_id}"', rpc)
+    ele = to_ele(rpc)
+    ele.attrib["message-id"] = message_id
+    return to_xml(ele)
